@@ -5,9 +5,9 @@ import sys
 import regex
 
 from . import plugins
-from . import files
 from . import utils
-from .plugin import get_plugins, load_plugins_by_package, DatasetPlugin, WritingPlugin, ValidationPlugin, SchemaPlugin
+from .plugin import get_plugins, load_plugins_by_package, DatasetPlugin, WritingPlugin, ValidationPlugin, SchemaPlugin, \
+    FileHandlerPlugin
 from .plugins.plugin_query import BoolExpr, Select, EqExpr, AnyExpr, AllExpr, ReExpr, CustomOpExpr, \
     EntityExpr
 
@@ -17,8 +17,7 @@ ENTITIES_PATTERN = regex.compile(r'(([^\W_]+)-([^\W_]+)_)+([^\W_]+)((\.[^\W_]+)+
 
 
 def load_dataset(base_dir: str):
-    """
-    Loads a dataset given its directory path on the file system.
+    """Loads a dataset given its directory path on the file system.
 
     .. code-block::
 
@@ -26,8 +25,15 @@ def load_dataset(base_dir: str):
         dataset_path = 'path/to/your/dataset'
         dataset = load_dataset(dataset_path)
 
-    :param base_dir: the dataset path to load from
-    :return: an object instance of type :py:class:`ancpbids.model.Dataset` which represents the dataset as an in-memory graph
+    Parameters
+    ----------
+    base_dir:
+        the dataset path to load from
+
+    Returns
+    -------
+    str
+        an object instance of type :py:class:`ancpbids.model.Dataset` which represents the dataset as an in-memory graph
     """
     schema = load_schema(base_dir)
     ds = schema.Dataset()
@@ -40,9 +46,26 @@ def load_dataset(base_dir: str):
 
 
 def load_schema(base_dir):
+    """Loads a BIDS schema object which represents the static/formal definition of the BIDS specification.
+
+    As per BIDS spec, a BIDS compliant dataset must have a BIDSVersion field in the dataset_description.json
+    file at the top level. This field is used to determine which BIDS schema to load.
+
+    In case the BIDSVersion field is missing or not supported, the earliest supported schema will be returned.
+
+    Parameters
+    ----------
+    base_dir:
+        The dataset directory path where a dataset_description.json must be located.
+
+    Returns
+    -------
+    object
+        A BIDS schema object which represents the static/formal definition of the BIDS specification.
+    """
     ds_descr_path = os.path.join(base_dir, "dataset_description.json")
     if os.path.exists(ds_descr_path):
-        ds_descr = files.load_contents(ds_descr_path)
+        ds_descr = utils.load_contents(ds_descr_path)
         if isinstance(ds_descr, dict) and 'BIDSVersion' in ds_descr:
             schema_version = ds_descr['BIDSVersion']
             schema_version = schema_version.replace('.', '_')
@@ -55,15 +78,28 @@ def load_schema(base_dir):
     return model_v1_7_0
 
 
-def save_dataset(ds, target_dir: str, context_folder=None):
+def save_dataset(ds: object, target_dir: str, context_folder=None):
+    """Copies the dataset graph into the provided target directory.
+
+    EXPERIMENTAL/UNSTABLE
+
+    Parameters
+    ----------
+    ds:
+        the dataset graph to save
+    target_dir
+        the target directory to save to
+    context_folder
+        a folder node within the dataset graph to limit to
+
+    """
     dataset_plugins = get_plugins(WritingPlugin)
     for dsplugin in dataset_plugins:
         dsplugin.execute(ds, target_dir, context_folder=context_folder)
 
 
 def validate_dataset(dataset) -> ValidationPlugin.ValidationReport:
-    """
-    Validates a dataset and returns a report object containing any detected validation errors.
+    """Validates a dataset and returns a report object containing any detected validation errors.
 
     Example:
 
@@ -75,9 +111,15 @@ def validate_dataset(dataset) -> ValidationPlugin.ValidationReport:
         if report.has_errors():
             raise "The dataset contains validation errors, cannot continue".
 
+    Parameters
+    ----------
+    dataset:
+        the dataset to validate
 
-    :param dataset: the dataset to validate
-    :return: a report object containing any detected validation errors or warning
+    Returns
+    -------
+    ValidationPlugin.ValidationReport
+        a report object containing any detected validation errors or warning
     """
     return _internal_validate_dataset(dataset)
 
@@ -94,6 +136,16 @@ def _internal_validate_dataset(dataset, plugin_acceptor=None):
 
 
 def write_derivative(ds, derivative):
+    """Writes the provided derivative folder to the dataset.
+    Note that a 'derivatives' folder will be created if not present.
+
+    Parameters
+    ----------
+    ds:
+        the dataset object to extend
+    derivative:
+        the derivative folder to write
+    """
     save_dataset(ds, target_dir=ds.get_absolute_path(), context_folder=derivative)
 
 
@@ -107,6 +159,10 @@ for pl in get_plugins(SchemaPlugin):
     for model in [model_v1_7_0, model_v1_7_1]:
         pl.execute(model)
 
+# load file handler plugins
+for pl in get_plugins(FileHandlerPlugin):
+    pl.execute(utils.FILE_READERS, utils.FILE_WRITERS)
+
 from .pybids_compat import BIDSLayout
 
 select = Select
